/*
  hppWorldModelPlanner.h.cpp -

  Copyright 2008 Centre National de la Recherche Scientifique (CNRS)

  Authors: Alireza Nakhaei,

*/
/*****************************************
 INCLUDES
*******************************************/


#ifndef HPP_WORLDMODEL_PLANNER_H
#define HPP_WORLDMODEL_PLANNER_H

/*************************************
INCLUDE
**************************************/
#include <iostream>
#include <fstream>

#include "hppCore/hppPlanner.h"
#include "hppModel/hppDevice.h"

#include "KineoWorks2/kwsPathOptimizer.h"
#include "KineoWorks2/kwsRoadmapBuilder.h"
#include "KineoWorks2/kwsConfig.h"
#include "KineoWorks2/kwsDirectPath.h"
#include "KineoWorks2/kwsSMLinear.h"

#include "KineoModel/kppRotationJointComponent.h"
#include "KineoModel/kppFreeFlyerJointComponent.h"
#include "KineoModel/kppAnchorJointComponent.h"
#include "KineoModel/kppJointComponent.h"
#include "KineoModel/kppSolidComponentRef.h"
#include "KineoKCDModel/kppKCDPolyhedron.h"
#include "KineoKCDModel/kppKCDAssembly.h"
#include "KineoWorks2/kwsPickerBasic.h"
#include "KineoWorks2/kwsPickerSmallestTree.h"
#include "KineoWorks2/kwsShooterConfigSpace.h"
#include "KineoWorks2/kwsIPPRdmBuilder.h"
#include "KineoWorks2/kwsRoadmap.h"

#include "hppModel/hppBody.h"
#include "hppWalkPlanner/hppWalkPlanner.h"

#include <time.h>
#include <sys/time.h>

#include "hppDynamicObstacle/hppPolyhedronMap.h"
#include "hppDynamicObstacle/hppDynamicObstacleOccupancyMap.h"
#include "hppDynamicObstacle/hppDynamicObstacleOccupancyMapVRML.h"
#include "hppDynamicObstacle/hppDynamicEnvHandler.h"

#include "kwsPlusRoadmap.h"

#include "hppWorldModelPlanner/hppNextBestViewPlanner.h"


/*************************************
CLASS
**************************************/

/**
   \brief Implement a two step walk planning strategy.

   The motion planning strategy is based on an iterative method that plan successive motions. The 3D environment which is generated in worldModelGrid3D  is  charged in attDynamicObstacle and then the assembly of obstacle and unknown volume will be used to present the obstacle and unknown area in the planning motion problem. In a first step, a path is computed for the bounding box from the initial configuration to goal configuration just by considering obstacle volume. Then, Unknown volume is added and the generated path in the last step will be validated by considering obstacles and unknown volume. In the last step, a path is generated from initial configuration and the last validated point and of the robot and then this path is animated using JRL walk pattern generator. 	*/


class ChppWorldModelPlanner : public ChppWalkPlanner
{
	public:
		/**
		 * \brief Constructor

	 	 * @param inStepLength Abscissa of the landing foot in the frame of the support foot in straight line motions.
	 	 * @param inStepSpace Absolute value of the ordinate of the landing foot in the frame of the support foot
	 	 * @param inSecurityDistance Distance above which centers of feet should remain in two consecutive steps.
	 	 * @param inModel Path to VRML model of HRP2.
		 */
		ChppWorldModelPlanner ( double inStepLength, double inStepSpace, double inSecurityDistance,
		                        double inBoxLateralTolerance, const char *inModel ) ;

		/**
		  	\brief Destructor
		 */
		~ChppWorldModelPlanner() ;

		/** \brief Set the attResolution. */
		ktStatus setResolution ( double inResolution );

		/** \brief Allocate the volume needed for the map. we should call the setResolution befor Allocate the map. */
		ktStatus allocateMap ( double xstart, double ystart, double zstart,double xend, double yend, double zend );

		/** \brief Initialize a  Planning problem.
		/Compute the bounding box of the first robot in problem vector and create a robot with this box.
		*/
		ktStatus initializeProblem();

		/** \brief Update the environmet by entering a vector of char which it elements are :Free "0" , Solid "1", Unknown "2". */
		ktStatus updateEnvironmentKppPolyhedron ( const std::vector<char> inRangeMap );

		/** \brief Solve problem*/
		ktStatus solve();

		/** \brief Clear Roadmap */
		ktStatus clearRoadmap();

		/** \brief Load Occupid environment */
		ktStatus loadOccupiedEnv();

		/** \brief Load complete environment */
		ktStatus loadCompleteEnv();

		/** \brief remove environment */
		ktStatus removeEnv();

		/** \brief update roadmap */
		ktStatus updateRoadmap();

		/** \brief Return the next accessible best view configuration */
		std::vector<double> nextBextView ( double initX, double initY, double initTheta,double goalX, double goalY, double goalTheta );

		/** \brief Return the visibility of a point in the model */
		double visibility ( double inX, double inY, double inTheta, double goalX, double goalY, double goalTheta );


	private:


		/** \brief Remove all the paths in the planner */
		ktStatus clearPaths();

		/** \brief  Map Volume */
		volume3D att3DMapArea;

		/** \brief  Next best view planner */
		ChppNextBestViewPlanner  *attNextBestViewPlanner ;

		/** \brief  A hppPolyhedronMap of occupied environment.*/
		ChppPolyhedronMapShPtr attOccupiedEnv;
		ChppDynamicObstacleOccupancyMapShPtr attDynamicObstacle;

		/** \brief  A hppPolyhedronMap of unknown environment.*/
		ChppPolyhedronMapShPtr attUnknownEnv;
		ChppDynamicObstacleOccupancyMapShPtr attDynamicUnknownEnv;

		/** \brief  A Dynamic environment handler.*/
		ChppDynamicEnvHandlerShPtr attDynamicEnvHandler;

		/** \brief for measuring th erequaired time for each process */
		struct timeval *Tps, *Tpf;
		struct timezone *Tzp;

};
#endif
